@@ -371,7 +371,7 @@ void print_peer_info(std::string& out, std::vector<libtorrent::peer_info> const&
 #endif
 
 		snprintf(str, sizeof(str)
-			, "%s%s (%s|%s) %s%s (%s|%s) %s%3d (%3d) %3d %c%c%c%c%c%c%c%c%c%c%c%c%c%c %c%c%c%c%c%c "
+			, "%s%s (%s|%s) %s%s (%s|%s) %s%3d (%3d) %3d %c%c%c%c%c%c%c%c%c%c%c%c%c%c%c %c%c%c%c%c%c "
 			, esc("32"), add_suffix(i->down_speed, "/s").c_str()
 			, add_suffix(i->total_download).c_str(), add_suffix(i->download_rate_peak, "/s").c_str()
 			, esc("31"), add_suffix(i->up_speed, "/s").c_str(), add_suffix(i->total_upload).c_str()
@@ -396,6 +396,7 @@ void print_peer_info(std::string& out, std::vector<libtorrent::peer_info> const&
 				(i->write_state == peer_info::bw_network)?'W':'.'
 			, (i->flags & peer_info::snubbed)?'S':'.'
 			, (i->flags & peer_info::upload_only)?'U':'D'
+			, (i->flags & peer_info::endgame_mode)?'-':'.'
 #ifndef TORRENT_DISABLE_ENCRYPTION
 			, (i->flags & peer_info::rc4_encrypted)?'E':
 				(i->flags & peer_info::plaintext_encrypted)?'e':'.'
@@ -1044,7 +1045,7 @@ int main(int argc, char* argv[])
 
 	for (;;)
 	{
-		char c;
+		char c = 0;
 		while (sleep_and_input(&c, refresh_delay))
 		{
 			if (c == 27)
