@@ -194,6 +194,7 @@ namespace libtorrent
 		, bool loopback)
 		: m_multicast_endpoint(multicast_endpoint)
 		, m_on_receive(handler)
+		, m_ip_broadcast(false)
 	{
 		TORRENT_ASSERT(is_multicast(m_multicast_endpoint.address()));
 
@@ -225,7 +226,23 @@ namespace libtorrent
 //				, print_address(multicast_endpoint.address()).c_str()
 //				, ec.message().c_str());
 #endif
-			open_unicast_socket(ios, i->interface_address);
+			open_unicast_socket(ios, i->interface_address
+				, i->netmask.is_v4() ? i->netmask.to_v4() : address_v4());
+		}
+	}
+
+	void broadcast_socket::enable_ip_broadcast(bool e)
+	{
+		if (e == m_ip_broadcast) return;
+		m_ip_broadcast = e;
+
+		asio::socket_base::broadcast option(m_ip_broadcast);
+		error_code ec;
+		for (std::list<socket_entry>::iterator i = m_unicast_sockets.begin()
+			, end(m_unicast_sockets.end()); i != end; ++i)
+		{
+			if (i->socket) continue;
+			i->socket->set_option(option, ec);
 		}
 	}
 
@@ -250,10 +267,11 @@ namespace libtorrent
 		m_sockets.push_back(socket_entry(s));
 		socket_entry& se = m_sockets.back();
 		s->async_receive_from(asio::buffer(se.buffer, sizeof(se.buffer))
-			, se.remote, bind(&broadcast_socket::on_receive, this, &se, _1, _2));
+			, se.remote, boost::bind(&broadcast_socket::on_receive, this, &se, _1, _2));
 	}
 
-	void broadcast_socket::open_unicast_socket(io_service& ios, address const& addr)
+	void broadcast_socket::open_unicast_socket(io_service& ios, address const& addr
+		, address_v4 const& mask)
 	{
 		using namespace asio::ip::multicast;
 		error_code ec;
@@ -262,10 +280,10 @@ namespace libtorrent
 		if (ec) return;
 		s->bind(udp::endpoint(addr, 0), ec);
 		if (ec) return;
-		m_unicast_sockets.push_back(socket_entry(s));
+		m_unicast_sockets.push_back(socket_entry(s, mask));
 		socket_entry& se = m_unicast_sockets.back();
 		s->async_receive_from(asio::buffer(se.buffer, sizeof(se.buffer))
-			, se.remote, bind(&broadcast_socket::on_receive, this, &se, _1, _2));
+			, se.remote, boost::bind(&broadcast_socket::on_receive, this, &se, _1, _2));
 	}
 
 	void broadcast_socket::send(char const* buffer, int size, error_code& ec)
@@ -296,6 +314,9 @@ namespace libtorrent
 			if (!i->socket) continue;
 			error_code e;
 			i->socket->send_to(asio::buffer(buffer, size), m_multicast_endpoint, 0, e);
+			if (m_ip_broadcast && i->socket->local_endpoint(e).address().is_v4())
+				i->socket->send_to(asio::buffer(buffer, size)
+					, udp::endpoint(i->broadcast_address(), m_multicast_endpoint.port()), 0, e);
 #ifndef NDEBUG
 //			extern std::string print_address(address const& addr);
 //			extern std::string print_endpoint(udp::endpoint const& ep);
@@ -320,13 +341,13 @@ namespace libtorrent
 		m_on_receive(s->remote, s->buffer, bytes_transferred);
 		if (!s->socket) return;
 		s->socket->async_receive_from(asio::buffer(s->buffer, sizeof(s->buffer))
-			, s->remote, bind(&broadcast_socket::on_receive, this, s, _1, _2));
+			, s->remote, boost::bind(&broadcast_socket::on_receive, this, s, _1, _2));
 	}
 
 	void broadcast_socket::close()
 	{
-		std::for_each(m_sockets.begin(), m_sockets.end(), bind(&socket_entry::close, _1));
-		std::for_each(m_unicast_sockets.begin(), m_unicast_sockets.end(), bind(&socket_entry::close, _1));
+		std::for_each(m_sockets.begin(), m_sockets.end(), boost::bind(&socket_entry::close, _1));
+		std::for_each(m_unicast_sockets.begin(), m_unicast_sockets.end(), boost::bind(&socket_entry::close, _1));
 
 		m_on_receive.clear();
 	}
