@@ -352,6 +352,27 @@ namespace libtorrent
 			}
 		}
 
+		// if this function is used with an older version of the library
+		// this member won't exist
+		// TODO: for distributions that needs to be binary compatible with
+		// previos 0.15.x releases, this if-block should be removed
+		if (!m_http_seeds.empty())
+		{
+			if (m_http_seeds.size() == 1)
+			{
+				dict["httpseeds"] = m_http_seeds.front();
+			}
+			else
+			{
+				entry& list = dict["httpseeds"];
+				for (std::vector<std::string>::const_iterator i
+					= m_http_seeds.begin(); i != m_http_seeds.end(); ++i)
+				{
+					list.list().push_back(entry(*i));
+				}
+			}
+		}
+
 		entry& info = dict["info"];
 		if (m_info_dict.type() == entry::dictionary_t)
 		{
@@ -495,7 +516,6 @@ namespace libtorrent
 	{
 		m_urls.push_back(announce_entry(url, tier));
 
-		using boost::bind;
 		std::sort(m_urls.begin(), m_urls.end()
 			, boost::bind(&announce_entry::second, _1) < boost::bind(&announce_entry::second, _2));
 	}
@@ -517,6 +537,11 @@ namespace libtorrent
 		m_url_seeds.push_back(url);
 	}
 
+	void create_torrent::add_http_seed(std::string const& url)
+	{
+		m_http_seeds.push_back(url);
+	}
+
 	void create_torrent::set_comment(char const* str)
 	{
 		m_comment = str;
