@@ -588,7 +588,9 @@ namespace libtorrent
 		if (insize != 0) return s;
 		// not sure why this would happen, but it seems to be possible
 		if (outsize > s.size() * 4) return s;
-		ret.resize(outsize);
+		// outsize is the number of bytes unused of the out-buffer
+		TORRENT_ASSERT(ret.size() >= outsize);
+		ret.resize(ret.size() - outsize);
 		return ret;
 	}
 #endif
