@@ -472,7 +472,7 @@ namespace libtorrent
 		{
 			for (int k = 0; k < i->iov_len; k += m_page_size)
 			{
-				cur_seg->Buffer = ((char*)i->iov_base) + k;
+				cur_seg->Buffer = PtrToPtr64((((char*)i->iov_base) + k));
 				++cur_seg;
 			}
 		}
@@ -493,6 +493,7 @@ namespace libtorrent
 			DWORD last_error = GetLastError();
 			if (last_error != ERROR_IO_PENDING)
 			{
+				TORRENT_ASSERT(GetLastError() != ERROR_BAD_ARGUMENTS);
 				ec = error_code(GetLastError(), get_system_category());
 				CloseHandle(ol.hEvent);
 				return -1;
@@ -655,7 +656,7 @@ namespace libtorrent
 		{
 			for (int k = 0; k < i->iov_len; k += m_page_size)
 			{
-				cur_seg->Buffer = ((char*)i->iov_base) + k;
+				cur_seg->Buffer = PtrToPtr64((((char*)i->iov_base) + k));
 				++cur_seg;
 			}
 		}
@@ -690,6 +691,7 @@ namespace libtorrent
 		{
 			if (GetLastError() != ERROR_IO_PENDING)
 			{
+				TORRENT_ASSERT(GetLastError() != ERROR_BAD_ARGUMENTS);
 				ec = error_code(GetLastError(), get_system_category());
 				CloseHandle(ol.hEvent);
 				return -1;
@@ -932,7 +934,7 @@ namespace libtorrent
 			}
 		}
 #endif // _WIN32_WINNT >= 0x501
-#else
+#else // NON-WINDOWS
 		struct stat st;
 		if (fstat(m_fd, &st) != 0)
 		{
@@ -980,9 +982,9 @@ namespace libtorrent
 			// way. If fallocate failed with some other error, it
 			// probably means the user should know about it, error out
 			// and report it.
-			if (errno != ENOSYS)
+			if (errno != ENOSYS && errno != EOPNOTSUPP)
 			{
-				ec.assign(ret, get_posix_category());
+				ec.assign(errno, get_posix_category());
 				return false;
 			}
 #endif // TORRENT_LINUX
@@ -993,7 +995,9 @@ namespace libtorrent
 			// if you get a compile error here, you might want to
 			// define TORRENT_HAS_FALLOCATE to 0.
 			ret = posix_fallocate(m_fd, 0, s);
-			if (ret != 0)
+			// posix_allocate fails with EINVAL in case the underlying
+			// filesystem does bot support this operation
+			if (ret != 0 && ret != EINVAL)
 			{
 				ec = error_code(ret, get_posix_category());
 				return false;
