@@ -159,11 +159,7 @@ namespace libtorrent { namespace
 
 			if (!have_all) return false;
 
-			hasher h;
-			h.update(&m_metadata[0], m_metadata_size);
-			sha1_hash info_hash = h.final();
-
-			if (info_hash != m_torrent.torrent_file().info_hash())
+			if (!m_torrent.set_metadata(&m_metadata[0], m_metadata_size))
 			{
 				std::fill(
 					m_have_metadata.begin()
@@ -171,26 +167,6 @@ namespace libtorrent { namespace
 					, false);
 				m_metadata_progress = 0;
 				m_metadata_size = 0;
-
-				if (m_torrent.alerts().should_post<metadata_failed_alert>())
-				{
-					m_torrent.alerts().post_alert(metadata_failed_alert(
-						m_torrent.get_handle()));
-				}
-
-				return false;
-			}
-
-			lazy_entry e;
-			lazy_bdecode(m_metadata.get(), m_metadata.get() + m_metadata_size, e);
-			std::string error;
-			if (!m_torrent.set_metadata(e, error))
-			{
-				// this means the metadata is correct, since we
-				// verified it against the info-hash, but we
-				// failed to parse it. Pause the torrent
-				// TODO: Post an alert!
-				m_torrent.pause();
 				return false;
 			}
 
@@ -231,6 +207,8 @@ namespace libtorrent { namespace
 				metadata();
 		}
 
+		int metadata_size() const { return m_metadata_size; }
+
 	private:
 		torrent& m_torrent;
 
@@ -333,8 +311,7 @@ namespace libtorrent { namespace
 			// abort if the peer doesn't support the metadata extension
 			if (m_message_index == 0) return;
 
-			// only send metadata if the torrent is non-private
-			if (m_torrent.valid_metadata() && !m_torrent.torrent_file().priv())
+			if (m_torrent.valid_metadata())
 			{
 				std::pair<int, int> offset
 					= req_to_offset(req, (int)m_tp.metadata().left());
@@ -392,7 +369,7 @@ namespace libtorrent { namespace
 
 			if (length > 500 * 1024)
 			{
-				m_pc.disconnect("LT_metadata message larger than 500 kB");
+				m_pc.disconnect(errors::metadata_too_large, 2);
 				return true;
 			}
 
@@ -417,7 +394,7 @@ namespace libtorrent { namespace
 					if (length != 3)
 					{
 						// invalid metadata request
-						m_pc.disconnect("invalid metadata request");
+						m_pc.disconnect(errors::invalid_metadata_request, 2);
 						return true;
 					}
 
@@ -442,22 +419,22 @@ namespace libtorrent { namespace
 
 					if (total_size > 500 * 1024)
 					{
-						m_pc.disconnect("metadata size larger than 500 kB");
+						m_pc.disconnect(errors::metadata_too_large, 2);
 						return true;
 					}
 					if (total_size <= 0)
 					{
-						m_pc.disconnect("invalid metadata size");
+						m_pc.disconnect(errors::invalid_metadata_size, 2);
 						return true;
 					}
 					if (offset > total_size || offset < 0)
 					{
-						m_pc.disconnect("invalid metadata offset");
+						m_pc.disconnect(errors::invalid_metadata_offset, 2);
 						return true;
 					}
 					if (offset + data_size > total_size)
 					{
-						m_pc.disconnect("invalid metadata message");
+						m_pc.disconnect(errors::invalid_metadata_message, 2);
 						return true;
 					}
 
@@ -485,9 +462,7 @@ namespace libtorrent { namespace
 				break;
 			default:
 				{
-					std::stringstream msg;
-					msg << "unknown metadata extension message: " << type;
-					m_pc.disconnect(msg.str().c_str());
+					m_pc.disconnect(errors::invalid_metadata_message, 2);
 				}
 			}
 			return true;
@@ -618,6 +593,8 @@ namespace libtorrent
 
 	boost::shared_ptr<torrent_plugin> create_metadata_plugin(torrent* t, void*)
 	{
+		// don't add this extension if the torrent is private
+		if (t->valid_metadata() && t->torrent_file().priv()) return boost::shared_ptr<torrent_plugin>();
 		return boost::shared_ptr<torrent_plugin>(new metadata_plugin(*t));
 	}
 
