@@ -136,7 +136,7 @@ namespace libtorrent
 		{
 			std::vector<downloading_piece>::const_iterator piece = std::find_if(
 				m_downloads.begin(), m_downloads.end()
-				, bind(&downloading_piece::index, _1) == index);
+				, boost::bind(&downloading_piece::index, _1) == index);
 			TORRENT_ASSERT(piece != m_downloads.end());
 			st = *piece;
 			st.info = 0;
@@ -186,7 +186,7 @@ namespace libtorrent
 	{
 		std::vector<downloading_piece>::iterator other = std::find_if(
 			m_downloads.begin(), m_downloads.end()
-			, bind(&downloading_piece::info, _1)
+			, boost::bind(&downloading_piece::info, _1)
 			== &m_block_info[(m_downloads.size() - 1) * m_blocks_per_piece]);
 		TORRENT_ASSERT(other != m_downloads.end());
 
@@ -2090,7 +2090,7 @@ namespace libtorrent
 			*j = i->peer_count + m_seeds;
 	}
 
-	void piece_picker::mark_as_writing(piece_block block, void* peer)
+	bool piece_picker::mark_as_writing(piece_block block, void* peer)
 	{
 #ifdef TORRENT_EXPENSIVE_INVARIANT_CHECKS
 		TORRENT_PIECE_PICKER_INVARIANT_CHECK;
@@ -2134,7 +2134,10 @@ namespace libtorrent
 			info.peer = peer;
 			if (info.state == block_info::state_requested) --i->requested;
 			TORRENT_ASSERT(i->requested >= 0);
-			TORRENT_ASSERT(info.state != block_info::state_writing);
+			if (info.state == block_info::state_writing
+				|| info.state == block_info::state_finished)
+				return false;
+
 			++i->writing;
 			info.state = block_info::state_writing;
 
@@ -2150,6 +2153,7 @@ namespace libtorrent
 			}
 			sort_piece(i);
 		}
+		return true;
 	}
 
 	void piece_picker::write_failed(piece_block block)
