@@ -149,7 +149,8 @@ namespace libtorrent
 			- (int)c.request_queue().size();
 
 #ifdef TORRENT_VERBOSE_LOGGING
-		(*c.m_logger) << time_now_string() << " PIECE_PICKER [ req: " << num_requests << " ]\n";
+		(*c.m_logger) << time_now_string() << " PIECE_PICKER [ req: " << num_requests
+			<< " endgame: " << c.endgame() << "]\n";
 #endif
 		TORRENT_ASSERT(c.desired_queue_size() > 0);
 		// if our request queue is already full, we
@@ -254,9 +255,20 @@ namespace libtorrent
 			TORRENT_ASSERT(p.num_peers(*i) == 0);
 
 			// don't request pieces we already have in our request queue
+			// This happens when pieces time out or the peer sends us
+			// pieces we didn't request. Those aren't marked in the
+			// piece picker, but we still keep track of them in the
+			// download queue
 			if (std::find_if(dq.begin(), dq.end(), has_block(*i)) != dq.end()
 				|| std::find_if(rq.begin(), rq.end(), has_block(*i)) != rq.end())
+			{
+#ifdef TORRENT_DEBUG
+				std::vector<pending_block>::const_iterator j
+					= std::find_if(dq.begin(), dq.end(), has_block(*i));
+				if (j != dq.end()) TORRENT_ASSERT(j->timed_out || j->not_wanted);
+#endif
 				continue;
+			}
 
 			// ok, we found a piece that's not being downloaded
 			// by somebody else. request it from this peer
@@ -267,12 +279,27 @@ namespace libtorrent
 			num_requests--;
 		}
 
+		// we have picked as many blocks as we should
+		// we're done!
+		if (num_requests <= 0)
+		{
+			// since we could pick as many blocks as we
+			// requested without having to resort to picking
+			// busy ones, we're not in end-game mode
+			c.set_endgame(false);
+			return;
+		}
+
+		// we did not pick as many pieces as we wanted, because
+		// there aren't enough. This means we're in end-game mode
+		// as long as we have at least one request outstanding,
+		// we shouldn't pick another piece
+		c.set_endgame(true);
+	
 		// if we don't have any potential busy blocks to request
-		// or if we have picked as many blocks as we should
 		// or if we already have outstanding requests, don't
 		// pick a busy piece
 		if (busy_pieces.empty()
-			|| num_requests <= 0
 			|| dq.size() + rq.size() > 0)
 		{
 			return;
@@ -533,6 +560,10 @@ namespace libtorrent
 		aux::session_impl const& ses = m_torrent->session();
 		if (ses.m_port_filter.access(p.port) & port_filter::blocked)
 			return false;
+
+		if (p.port < 1024)
+			return false;
+
 		return true;
 	}
 
@@ -884,7 +915,7 @@ namespace libtorrent
 			std::pair<iterator, iterator> range = find_peers(remote.address());
 			iterator i = std::find_if(range.first, range.second
 				, match_peer_endpoint(remote));
-			if (i != m_peers.end())
+			if (i != range.second)
 			{
 				policy::peer& pp = **i;
 				if (pp.connection)
@@ -977,6 +1008,13 @@ namespace libtorrent
 			return 0;
 		}
 
+		if (remote.port() < 1024)
+		{
+			if (ses.m_alerts.should_post<peer_blocked_alert>())
+				ses.m_alerts.post_alert(peer_blocked_alert(remote.address()));
+			return 0;
+		}
+
 		// if the IP is blocked, don't add it
 		if (ses.m_ip_filter.access(remote.address()) & ip_filter::blocked)
 		{
@@ -1190,7 +1228,6 @@ namespace libtorrent
 		TORRENT_ASSERT(!is_connect_candidate(*p, m_finished));
 
 		// save transfer rate limits
-		int rate_limit;
 		p->upload_rate_limit = c.upload_limit();
 		p->download_rate_limit = c.download_limit();
 
