@@ -218,10 +218,12 @@ void udp_socket::on_read(udp::socket* s, error_code const& e, std::size_t bytes_
 		try {
 #endif
 
-		if (m_tunnel_packets && m_v4_ep == m_proxy_addr)
+		if (m_tunnel_packets)
 		{
 			l.unlock();
-			unwrap(e, m_v4_buf, bytes_transferred);
+			// if the source IP doesn't match the proxy's, ignore the packet
+			if (m_v4_ep == m_proxy_addr)
+				unwrap(e, m_v4_buf, bytes_transferred);
 		}
 		else
 		{
@@ -246,10 +248,12 @@ void udp_socket::on_read(udp::socket* s, error_code const& e, std::size_t bytes_
 		try {
 #endif
 
-		if (m_tunnel_packets && m_v6_ep == m_proxy_addr)
+		if (m_tunnel_packets)
 		{
 			l.unlock();
-			unwrap(e, m_v6_buf, bytes_transferred);
+			// if the source IP doesn't match the proxy's, ignore the packet
+			if (m_v6_ep == m_proxy_addr)
+				unwrap(e, m_v6_buf, bytes_transferred);
 		}
 		else
 		{
@@ -641,7 +645,7 @@ void udp_socket::socks_forward_udp(mutex_t::scoped_lock& l)
 	write_uint8(5, p); // SOCKS VERSION 5
 	write_uint8(3, p); // UDP ASSOCIATE command
 	write_uint8(0, p); // reserved
-	write_uint8(0, p); // ATYP IPv4
+	write_uint8(1, p); // ATYP IPv4
 	write_uint32(0, p); // IP any
 	write_uint16(m_bind_port, p);
 
@@ -700,6 +704,22 @@ void udp_socket::connect2(error_code const& e)
 		udp_socket::send(p.ep, &p.buf[0], p.buf.size(), ec);
 		m_queue.pop_front();
 	}
+
+	asio::async_read(m_socks5_sock, asio::buffer(m_tmp_buf, 10)
+		, boost::bind(&udp_socket::hung_up, this, _1));
+}
+
+void udp_socket::hung_up(error_code const& e)
+{
+	CHECK_MAGIC;
+	mutex_t::scoped_lock l(m_mutex);	
+
+	if (e == asio::error::operation_aborted || m_abort) return;
+
+	l.unlock();
+
+	// the socks connection was closed, re-open it
+	set_proxy_settings(m_proxy_settings);
 }
 
 rate_limited_udp_socket::rate_limited_udp_socket(io_service& ios
@@ -741,7 +761,7 @@ void rate_limited_udp_socket::on_tick(error_code const& e)
 	if (e) return;
 	if (is_closed()) return;
 	error_code ec;
-	ptime now = time_now();
+	ptime now = time_now_hires();
 	m_timer.expires_at(now + seconds(1), ec);
 	m_timer.async_wait(boost::bind(&rate_limited_udp_socket::on_tick, this, _1));
 
@@ -751,9 +771,10 @@ void rate_limited_udp_socket::on_tick(error_code const& e)
 
 	if (m_queue.empty()) return;
 
-	while (!m_queue.empty() && int(m_queue.front().buf.size()) >= m_quota)
+	while (!m_queue.empty() && int(m_queue.front().buf.size()) <= m_quota)
 	{
 		queued_packet const& p = m_queue.front();
+		TORRENT_ASSERT(m_quota >= p.buf.size());
 		m_quota -= p.buf.size();
 		error_code ec;
 		udp_socket::send(p.ep, &p.buf[0], p.buf.size(), ec);
