@@ -49,7 +49,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/session_impl.hpp"
 #include "libtorrent/parse_url.hpp"
 
-using boost::bind;
 using boost::shared_ptr;
 using libtorrent::aux::session_impl;
 
@@ -132,11 +131,12 @@ namespace libtorrent
 		piece_block_progress ret;
 
 		ret.piece_index = m_requests.front().piece;
-		ret.bytes_downloaded = m_block_pos;
-		if (m_block_pos)
-			ret.block_index = (m_requests.front().start + m_block_pos - 1) / t->block_size();
-		else
-			ret.block_index = (m_requests.front().start + m_block_pos) / t->block_size();
+		ret.bytes_downloaded = m_block_pos % t->block_size();
+		// this is used to make sure that the block_index stays within
+		// bounds. If the entire piece is downloaded, the block_index
+		// would otherwise point to one past the end
+		int correction = m_block_pos ? -1 : 0;
+		ret.block_index = (m_requests.front().start + m_block_pos + correction) / t->block_size();
 
 		ret.full_block_bytes = t->block_size();
 		const int last_piece = t->torrent_file().num_pieces() - 1;
@@ -382,16 +382,10 @@ namespace libtorrent
 				}
 
 				// if the status code is not one of the accepted ones, abort
-				if (m_parser.status_code() != 206 // partial content
-					&& m_parser.status_code() != 200 // OK
-					&& !(m_parser.status_code() >= 300 // redirect
-						&& m_parser.status_code() < 400))
+				if (!is_ok_status(m_parser.status_code()))
 				{
-					if (m_parser.status_code() == 503)
-					{
-						// temporarily unavailable, retry later
-						t->retry_web_seed(m_original_url, web_seed_entry::url_seed);
-					}
+					// temporarily unavailable, retry later
+					t->retry_web_seed(m_original_url, web_seed_entry::url_seed);
 					t->remove_web_seed(m_original_url, web_seed_entry::url_seed);
 					std::string error_msg = to_string(m_parser.status_code()).elems
 						+ (" " + m_parser.message());
@@ -426,7 +420,7 @@ namespace libtorrent
 					, end(headers.end()); i != end; ++i)
 					(*m_logger) << "   " << i->first << ": " << i->second << "\n";
 #endif
-				if (m_parser.status_code() >= 300 && m_parser.status_code() < 400)
+				if (is_redirect(m_parser.status_code()))
 				{
 					// this means we got a redirection request
 					// look for the location header
@@ -535,18 +529,22 @@ namespace libtorrent
 			peer_request front_request = m_requests.front();
 
 			TORRENT_ASSERT(m_block_pos >= 0);
-			if (m_block_pos + payload_transferred > front_request.length)
-				payload_transferred = front_request.length - m_block_pos;
 
+#ifdef TORRENT_VERBOSE_LOGGING
+			(*m_logger) << "*** payload_transferred: " << payload_transferred
+				<< " [" << front_request.piece << ":" << front_request.start
+				<< " = " << front_request.length <<  "]\n";
+#endif
 			m_statistics.received_bytes(payload_transferred, 0);
-			incoming_piece_fragment(payload_transferred);
 			bytes_transferred -= payload_transferred;
 			m_range_pos += payload_transferred;
 			m_block_pos += payload_transferred;
 			if (m_range_pos > range_end - range_start) m_range_pos = range_end - range_start;
 
-//			std::cerr << "REQUESTS: m_requests: " << m_requests.size()
-//				<< " file_requests: " << m_file_requests.size() << std::endl;
+#if 0
+			std::cerr << "REQUESTS: m_requests: " << m_requests.size()
+				<< " file_requests: " << m_file_requests.size() << std::endl;
+#endif
 
 			int file_index = m_file_requests.front();
 			peer_request in_range = info.orig_files().map_file(file_index, range_start
@@ -555,13 +553,13 @@ namespace libtorrent
 			size_type rs = size_type(in_range.piece) * info.piece_length() + in_range.start;
 			size_type re = rs + in_range.length;
 			size_type fs = size_type(front_request.piece) * info.piece_length() + front_request.start;
-/*
+#if 0
 			size_type fe = fs + front_request.length;
 
 			std::cerr << "RANGE: r = (" << rs << ", " << re << " ) "
 				"f = (" << fs << ", " << fe << ") "
 				"file_index = " << file_index << " received_body = " << m_received_body << std::endl;
-*/
+#endif
 
 			// the http response body consists of 3 parts
 			// 1. the middle of a block or the ending of a block
@@ -573,6 +571,8 @@ namespace libtorrent
 
 			if (!range_overlaps_request)
 			{
+				incoming_piece_fragment((std::min)(payload_transferred
+					, front_request.length - m_block_pos));
 				m_statistics.received_bytes(0, bytes_transferred);
 				// this means the end of the incoming request ends _before_ the
 				// first expected byte (fs + m_piece.size())
@@ -604,6 +604,7 @@ namespace libtorrent
 				m_body_start += copy_size;
 				TORRENT_ASSERT(m_received_body <= range_end - range_start);
 				TORRENT_ASSERT(int(m_piece.size()) <= front_request.length);
+				incoming_piece_fragment(copy_size);
 				if (int(m_piece.size()) == front_request.length)
 				{
 					// each call to incoming_piece() may result in us becoming
@@ -614,8 +615,7 @@ namespace libtorrent
 					incoming_piece(front_request, &m_piece[0]);
 					m_requests.pop_front();
 					if (associated_torrent().expired()) return;
-					TORRENT_ASSERT(m_block_pos == front_request.length);
-					m_block_pos = 0;
+					m_block_pos -= front_request.length;
 					cut_receive_buffer(m_body_start, t->block_size() + 1024);
 					m_body_start = 0;
 					recv_buffer = receive_buffer();
@@ -633,11 +633,11 @@ namespace libtorrent
 				peer_request r = m_requests.front();
 				TORRENT_ASSERT(recv_buffer.left() >= r.length);
 
+				incoming_piece_fragment(r.length);
 				incoming_piece(r, recv_buffer.begin);
 				m_requests.pop_front();
 				if (associated_torrent().expired()) return;
-				TORRENT_ASSERT(m_block_pos == front_request.length);
-				m_block_pos = 0;
+				m_block_pos -= r.length;
 				m_received_body += r.length;
 				TORRENT_ASSERT(receive_buffer().begin + m_body_start == recv_buffer.begin);
 				TORRENT_ASSERT(m_received_body <= range_end - range_start);
@@ -683,6 +683,7 @@ namespace libtorrent
 				continue;
 			}
 			if (bytes_transferred == 0) break;
+			TORRENT_ASSERT(payload_transferred > 0);
 		}
 		TORRENT_ASSERT(bytes_transferred == 0);
 	}
