@@ -36,24 +36,33 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/session.hpp"
 #include <boost/tuple/tuple.hpp>
 
+namespace libtorrent
+{
+	class alert;
+	struct add_torrent_params;
+}
 
-void print_alerts(libtorrent::session& ses, char const* name
+bool print_alerts(libtorrent::session& ses, char const* name
 	, bool allow_disconnects = false
 	, bool allow_no_torrents = false
-	, bool allow_failed_fastresume = false);
+	, bool allow_failed_fastresume = false
+	, bool (*)(libtorrent::alert*) = 0);
+
 void test_sleep(int millisec);
 
-boost::intrusive_ptr<libtorrent::torrent_info> create_torrent(std::ostream* file = 0, int piece_size = 16 * 1024, int num_pieces = 1024 / 8);
+boost::intrusive_ptr<libtorrent::torrent_info> create_torrent(std::ostream* file = 0, int piece_size = 16 * 1024, int num_pieces = 13);
 
-boost::tuple<libtorrent::torrent_handle, libtorrent::torrent_handle
+boost::tuple<libtorrent::torrent_handle
+	, libtorrent::torrent_handle
 	, libtorrent::torrent_handle>
 setup_transfer(libtorrent::session* ses1, libtorrent::session* ses2
 	, libtorrent::session* ses3, bool clear_files, bool use_metadata_transfer = true
 	, bool connect = true, std::string suffix = "", int piece_size = 16 * 1024
-	, boost::intrusive_ptr<libtorrent::torrent_info>* torrent = 0);
+	, boost::intrusive_ptr<libtorrent::torrent_info>* torrent = 0, bool super_seeding = false
+	, libtorrent::add_torrent_params const* p = 0);
 
-void start_web_server(int port, bool ssl = false);
-void stop_web_server(int port);
+int start_web_server(bool ssl = false);
+void stop_web_server();
 void start_proxy(int port, int type);
 void stop_proxy(int port);
 
